/** @file ccp_scen.cpp
 *
 * @author Alexander Kamkin (kamkin@ispras.ru)
 */

#include <ccp_scen.h>

namespace cpptesk {
namespace examples {
namespace ccp {

bool CacheCoherenceScenario::init(int argc, char **argv)
{
    UNUSED_ARG(argc);
    UNUSED_ARG(argv);

    model.initialize();
    return true;
}

void CacheCoherenceScenario::finish()
{
    model.finalize();
}

bool CacheCoherenceScenario::read(Ctx &ctx)
{
    IBEGIN
    for(IVAR(i) = 0; (unsigned)IVAR(i) < model.size(); IVAR(i)++)
    {
        IACTION
        {
            model.read(IVAR(i));
            YIELD(model.invariant());
        }
    }

    IEND
}

bool CacheCoherenceScenario::write(Ctx &ctx)
{
    IBEGIN

    for(IVAR(i) = 0; (unsigned)IVAR(i) < model.size(); IVAR(i)++)
    {
        IACTION
        {
            model.write(IVAR(i));
            YIELD(model.invariant());
        }
    }

    IEND
}

bool CacheCoherenceScenario::replace(Ctx &ctx)
{
    IBEGIN

    for(IVAR(i) = 0; (unsigned)IVAR(i) < model.size(); IVAR(i)++)
    {
        IACTION
        {
	    if(model.pre_replace(IVAR(i)))
            {
	        model.replace(IVAR(i));
            }
            YIELD(model.invariant());
        }
    }

    IEND
}

bool CacheCoherenceScenario::reset(Ctx &ctx)
{
    IBEGIN

    IACTION
    {
        model.reset();
	YIELD(model.invariant());
    }

    IEND
}

std::string CacheCoherenceScenario::getState()
{
    return model.getMultiSetState();
}

}}} // namespace cpptesk::examples::ccp

