`include  "myfifo.v"


module mul_fifo(
DO,
DO_VAL,
IS_EMPTY,
IS_FULL,

CLK,
RST,
DI,
RD_A,
WR_A,
CE_RD,
CE_WR

);


//------Parameters declaration:---------
    parameter  FCOUNT_SIZE = 2;
	parameter  ADDR_SIZE   = 4;
	parameter  DATA_SIZE   = 8;	
	parameter  SUPER_TEXT   = "Hello multy fifo!";	
//--------------------------------------

output [(DATA_SIZE-1):0] DO;      // Data Output
output DO_VAL; // Data Output Valid
output [((1 << FCOUNT_SIZE)-1):0] IS_EMPTY; // Fifo is empty
output [((1 << FCOUNT_SIZE)-1):0] IS_FULL; // Fifo is full

input  CLK;            // Clock
input  RST;            // Reset
input [(DATA_SIZE-1):0] DI;      // Data Input
input [(FCOUNT_SIZE-1):0] RD_A;        //Read Address
input [(FCOUNT_SIZE-1):0] WR_A;        //Write Address
input  CE_RD;            // Read Chip Select Enable
input  CE_WR;            // Write Chip Select Enable

reg  [(DATA_SIZE-1):0] DO; 
reg  DO_VAL;


wire [((1 << FCOUNT_SIZE)-1):0] CE_RD_tmp;
wire [((1 << FCOUNT_SIZE)-1):0] CE_WR_tmp;

wire [((1 << FCOUNT_SIZE)-1):0] DO_VAL_tmp;
wire [(DATA_SIZE-1):0] DO_tmp [((1 << FCOUNT_SIZE)-1):0];

always @(posedge CLK) begin
    
    if(RST) begin
        DO_VAL <= 1'b0;
    end else begin
        DO_VAL <=  |DO_VAL_tmp;
    end

end


generate
    genvar i;

    for (i=0; i!=(1 << FCOUNT_SIZE); i=i+1) begin
        myfifo #(ADDR_SIZE,DATA_SIZE) one_fifo(
        .DO(DO_tmp[i]),
        .DO_VAL(DO_VAL_tmp[i]),
        .IS_EMPTY(IS_EMPTY[i]),
        .IS_FULL(IS_FULL[i]),
        .CLK(CLK),
        .RST(RST),
        .DI(DI),
        .CE_RD(CE_RD_tmp[i]),
        .CE_WR(CE_WR_tmp[i])
        );
        
        assign CE_WR_tmp[i] = (WR_A == i) ? CE_WR : 1'b0;
        assign CE_RD_tmp[i] = (RD_A == i) ? CE_RD : 1'b0;

        always @(posedge CLK) begin
            if(DO_VAL_tmp[i]) begin
                DO <= DO_tmp[i];
            end
        end
        
    end

endgenerate

endmodule
