`include "myfifo.v"
`include "arbiter.v"

`timescale 1ns/100ps

module myfifo3parts (
DO,
DO_VAL,
IS_EMPTY0,
IS_EMPTY1,
IS_FULL0,
IS_FULL1,
CLK,
RST,
DI0,
DI1,
CE_RD,
CE_WR0,
CE_WR1
);

parameter WIDTH = 8;

output [WIDTH-1:0] DO;
output DO_VAL;
output IS_EMPTY0;
output IS_EMPTY1;
output IS_FULL0;
output IS_FULL1;
input CLK;
input RST;
input [WIDTH-1:0] DI0;
input [WIDTH-1:0] DI1;
input CE_RD;
input CE_WR0;
input CE_WR1;

wire [WIDTH-1:0] DO0_TO_ARBITER;
wire [WIDTH-1:0] DO1_TO_ARBITER;
wire [WIDTH-1:0] DO;
wire [WIDTH-1:0] DI_FROM_ARBITER;
wire DO_VAL0_TO_ARBITER, DO_VAL1_TO_ARBITER, CE_RD0_FROM_ARBITER, CE_RD1_FROM_ARBITER;

myfifo input1 (
    .DO(DO0_TO_ARBITER),
    .DO_VAL(DO_VAL0_TO_ARBITER),
    .IS_EMPTY(IS_EMPTY0),
    .IS_FULL(IS_FULL0),
    .CLK(CLK),
    .RST(RST),
    .DI(DI0),
    .CE_RD(CE_RD0_FROM_ARBITER),
    .CE_WR(CE_WR0)
);

myfifo input2 (
    .DO(DO1_TO_ARBITER),
    .DO_VAL(DO_VAL1_TO_ARBITER),
    .IS_EMPTY(IS_EMPTY1),
    .IS_FULL(IS_FULL1),
    .CLK(CLK),
    .RST(RST),
    .DI(DI1),
    .CE_RD(CE_RD1_FROM_ARBITER),
    .CE_WR(CE_WR1)
);

myfifo output3 (
    .DO(DO),
    .DO_VAL(DO_VAL),
    .IS_EMPTY(IS_EMPTY_TO_ARBITER),
    .IS_FULL(IS_FULL_TO_ARBITER),
    .CLK(CLK),
    .RST(RST),
    .DI(DI_FROM_ARBITER),
    .CE_RD(CE_RD),
    .CE_WR(CE_WR_FROM_ARBITER)
);

arbiter arb (
    .DI0(DO0_TO_ARBITER),
    .DI1(DO1_TO_ARBITER),
    .DI_VAL0(DO_VAL0_TO_ARBITER),
    .DI_VAL1(DO_VAL1_TO_ARBITER),
    .IS_EMPTY0(IS_EMPTY0),
    .IS_EMPTY1(IS_EMPTY1),
    .IS_EMPTY2(IS_EMPTY_TO_ARBITER),
    .IS_FULL0(IS_FULL0),
    .IS_FULL1(IS_FULL1),
    .IS_FULL2(IS_FULL_TO_ARBITER),
    .CLK(CLK),
    .RST(RST),
    .DO(DI_FROM_ARBITER),
    .CE_RD0(CE_RD0_FROM_ARBITER),
    .CE_RD1(CE_RD1_FROM_ARBITER),
    .CE_WR(CE_WR_FROM_ARBITER)
);

endmodule
