/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Sequence recognizer
///////////////////////////////////////////////////////////////////////////////

#include <hw/assertion.hpp>
#include <hw/sequence/recognizer.hpp>

namespace cpptesk {
namespace hw {
namespace sequence {

////////////////////////////////////////////////////////////////////////////////
// Sequence Recognizer
////////////////////////////////////////////////////////////////////////////////

bool SequenceRecognizer::cycle()
{
    bool new_match = false;
    SequenceRecognizer *recognizer = 0;

    if(isFinished())
        { return new_match; }
    if(delay.min > 0)
        { goto tick_recognizer; }
    if(getTime() > time_constraints.max)
        { goto stop_recognizer; }

    if(!timeout)
    {
        switch(sequence.getType())
        {
        case Sequence::EVENT:  { recognizer = new EventRecognizer (sequence); break; }
        case Sequence::GUARD:  { recognizer = new GuardRecognizer (sequence); break; }
        case Sequence::SEQ:    { recognizer = new SeqRecognizer   (sequence); break; }
        case Sequence::AND:    { recognizer = new AndRecognizer   (sequence); break; }
        case Sequence::OR:     { recognizer = new OrRecognizer    (sequence); break; }
        case Sequence::REPEAT: { recognizer = new RepeatRecognizer(sequence); break; }
        }
        CPPTESK_ASSERTION(recognizer, "Cannot find a recognizer: unknown sequence type");
        alt.push_back(recognizer);
    }

    for(std::list<SequenceRecognizer*>::iterator i = alt.begin(); i != alt.end(); i++)
    {
        new_match |= (*i)->onCycle();

	if((*i)->isFinished())
	    { delete (*i); alt.erase(i--); }
    }

    if(delay.max == 0)
    {
        if(alt.empty())
            { goto stop_recognizer; }
	else
            { timeout = true; }
    }

tick_recognizer:
    tick();
    return new_match;

stop_recognizer:
    stop();
    return new_match;
}

bool SequenceRecognizer::onCycle()
{
    return false;
}

void SequenceRecognizer::reset()
{
    start();

    delay = initial_delay;
    count = time = 0;

    timeout = false;

    if(time_constraints.min > delay.min)
        { delay.min = time_constraints.min; }
    if(time_constraints.max < delay.max)
        { delay.max = time_constraints.max; }
    if(delay.min > delay.max)
        { stop(); }

    for(std::list<SequenceRecognizer*>::iterator i = alt.begin(); i != alt.end(); i++)
        { delete *i; }

    alt.clear();
}

}}} // namespace cpptesk::hw::sequence

