/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>

#include <atl/list.h>
#include <generator/ndfsm.h>

static void print_path(TE_ArcList *path, const char* filename)
{
    int i;
    FILE *file = fopen(filename, "w");

    if(file == NULL) { return; }

    if(path == NULL)
    {
        fprintf(file, "NULL");
        goto exit_function;
    }

    for(i = 0; i < size_List(r(path)); i++)
    {
        String *arc_string;
        TE_ArcId *arc_id = get_List(r(path), i);

        assertion(arc_id != NULL, "print_path: arc_id == NULL");
        assertion(arc_id->start_node_id != NULL, "print_path: start_node_id == NULL");

        arc_string = toString(r(arc_id));
        fprintf(file, "%s\n", toCharArray_String(r(arc_string)));

        destroy(arc_string);
    }

exit_function:
    fclose(file);
}

static void testcase(const char *inputfile, const char *outputfile, int argc, char **argv)
{
    TE_Graph graph;
    TE_NodeId *start_node_id;
    TE_ArcList *path;

    init_Graph(&graph, TE_UPDATE_OFF, &TE_NO_SYNC_MUTEX);
    TE_NDFSM.init(&graph, argc, argv);

    fromFile_Default(&TE_NDFSM, &graph, inputfile);

    start_node_id = create_NodeId(create_String("start"));
    path = TE_NDFSM.getPath(&graph, start_node_id);
    print_path(path, outputfile);

    destroy(start_node_id);
    destroy(path);

    TE_NDFSM.finish(&graph);
}

int main(int argc, char **argv)
{
    testcase("testcases/graph000.xml", "testcases/graph000_path.out", argc, argv);
    testcase("testcases/graph001.xml", "testcases/graph001_path.out", argc, argv);
    testcase("testcases/graph002.xml", "testcases/graph002_path.out", argc, argv);
    testcase("testcases/graph003.xml", "testcases/graph003_path.out", argc, argv);
    testcase("testcases/graph004.xml", "testcases/graph004_path.out", argc, argv);
    testcase("testcases/graph005.xml", "testcases/graph005_path.out", argc, argv);

    return 0;
}
