/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <assert.h>
#include <string.h>

#include <utils/parparser.h>
#include <utils/testreg.h>

static TestScenario *local_initTestScenario;
static TestScenario *local_testScenario;
static TestEngine *local_testEngine;
static int local_argc;
static char **local_argv;
static int local_netscen;

extern start_scenario_with_wrapper(void(*wrapper)(void));

void set_scenario_wrapper(TestScenario *testScenario, TestEngine *testEngine, char *parameters, bool netscen)
{
    local_testScenario = testScenario;
    local_testEngine = testEngine;
    local_netscen = netscen;
    local_argc = parse_parameters(parameters, local_testScenario->name, &local_argv);
}

void set_scenario_wrapper_with_init(TestScenario *initTestScenario, TestScenario *testScenario, TestEngine *testEngine, char *parameters, bool netscen)
{
    local_initTestScenario = initTestScenario;
    set_scenario_wrapper(testScenario, testEngine, parameters, netscen);
}

void scenario_wrapper(void)
{
    if(!local_netscen)
        localmain(local_testEngine, local_testScenario, local_argc, local_argv);
    else
        netmain(local_testEngine, local_testScenario, local_argc, local_argv);
}

void scenario_wrapper_with_init(void)
{
    localmain(local_testEngine, local_initTestScenario, local_argc, local_argv);
    scenario_wrapper();
}

int find_scenario_TestRegistry(char *scen_name, char *scen_params)
{
    while(isblank(scen_name[0])) scen_name++;

    /* Erase blank symbols at the begin of scen_params */
    int position = 0;
    while(isblank(scen_params[position])) position++;
        memmove(scen_params, scen_params + position, strlen(scen_params) - position + 1);

    /* This code looks for item in test registry where scenario is named *scen.
       Don't forget to create the registry! */
    int i;
    for(i = 0; test_registry[i].testName; i++) {
        if(!strcmp(scen_name, test_registry[i].testName)) {
            int length_local = test_registry[i].testParams == NULL ? 0 : strlen(test_registry[i].testParams),
                length_global = strlen(scen_params);

            if(length_local > 0) {
                assert(length_local + length_global < SCEN_ARGV_LENGTH - 1);
                memmove(scen_params + length_local + 1, scen_params, length_global);
                memmove(scen_params, test_registry[i].testParams, length_local);
                scen_params[length_local] = ' ';
            }

            break;
        }
    }

    assert(test_registry[i].testName != NULL);
    assert(test_registry[i].testScenario != NULL);
    assert(test_registry[i].testEngine != NULL);
    
    return i;
}

int startscen_TestRegistry(bool netscen, char *scen_name, char *scen_params)
{
    int scen_number = find_scenario_TestRegistry(scen_name, scen_params);
  
    if(test_registry[scen_number].testType == CTESK) {
        TestScenario* ctest = (TestScenario*)test_registry[scen_number].testScenario;
        set_scenario_wrapper(ctest, test_registry[scen_number].testEngine, scen_params, netscen);
        return start_scenario_with_wrapper(scenario_wrapper);
    }
    else {
        assert(false);
        return 1;
    }
}

/*
int start_scenario(TestScenario *scen, char *params)
{
    set_scenario_wrapper(scen, &TE_NDFSM, params, 0);
    return start_scenario_with_wrapper(scenario_wrapper);
}

int start_net_scenario(TestScenario *scen, char *params)
{
    set_scenario_wrapper(scen, &TE_NDFSM, params, 1);
    return start_scenario_with_wrapper(scenario_wrapper);
}

int start_scenario_with_init(TestScenario *initscen, TestScenario *scen, char *params)
{
    set_scenario_wrapper_with_init(initscen, scen, &TE_NDFSM, params, 0);
    return start_scenario_with_wrapper(scenario_wrapper_with_init);
}

int start_net_scenario_with_init(TestScenario *initscen, TestScenario *scen, char *params)
{
    set_scenario_wrapper_with_init(initscen, scen, &TE_NDFSM, params, 1);
    return start_scenario_with_wrapper(scenario_wrapper_with_init);
}

int start_scenario_new(TestScenario *scen, TestEngine *testEngine, char *params, bool netscen)
{
    set_scenario_wrapper(scen, testEngine, params, netscen);
    return start_scenario_with_wrapper(scenario_wrapper);
}

int start_scenario_new_with_init(TestScenario *initscen, TestScenario *scen, TestEngine *testEngine, char *params, bool netscen)
{
    set_scenario_wrapper_with_init(initscen, scen, testEngine, params, netscen);
    return start_scenario_with_wrapper(scenario_wrapper_with_init);
}
*/

