#!/bin/perl -w

use POSIX;

my @topologies_list = ("ring", "torus", "full", "empty");
my %topologies_hash = map { $_ => 1 } @topologies_list;

my $usage =
    "Usage:\n".
    "  $0 topology nodes_number\n\n".
    "Options:\n".
    "  topology     - @topologies_list\n".
    "  nodes_number - number (>= 1)\n\n";

die "$0: incorrect number of arguments\n\n".$usage
unless (scalar @ARGV == 2);

my $topology     = $ARGV[0];
my $nodes_number = $ARGV[1];

die "$0: incorrect topology\n\n".$usage
unless (exists $topologies_hash{lc($topology)});

die "$0: incorrect number of nodes\n\n".$usage
unless ($nodes_number >= 1);

if($nodes_number < 2)
{
    $topology = "empty";
}

###############################################################################
# Ring: One-Dimensional Torus
###############################################################################

if(lc($topology) eq "ring")
{
    for(my $node = 1; $node < $nodes_number; $node++)
    {
        print $node, ":", $node + 1, "\n";
    }

    print $nodes_number, ":", 1, "\n";

    exit 0;
}

###############################################################################
# Torus: Two-Dimensional Torus
###############################################################################

if(lc($topology) eq "torus")
{
    my $torus_height = floor(sqrt($nodes_number));
    my $torus_width  = $torus_height;

    while($torus_height * $torus_width < $nodes_number)
    {
        $torus_width++;
    }

    for(my $node = 1; $node <= $nodes_number;  $node++)
    {
        # Make horizontal links
        if($node % $torus_width == 0)
        {
            print $node, ":", $node + 1 - $torus_width, "\n";
        }
        elsif($node < $nodes_number)
        {
            print $node, ":", $node + 1, "\n";
        }
        elsif(($torus_height - 1) * $torus_width + 1 != $node)
        {
            print $node, ":", ($torus_height - 1) * $torus_width + 1, "\n";
        }

        # Make vertical links
        if($node + $torus_width <= $nodes_number)
        {
            print $node, ":", $node + $torus_width, "\n";
        }
        elsif(($node - 1) % $torus_width + 1 != $node)
        {
            print $node, ":", ($node - 1) % $torus_width + 1, "\n";
        }
    }

    exit 0;
}

###############################################################################
# Full: Fully Connected Graph
###############################################################################

if(lc($topology) eq "full")
{
    for(my $node = 1; $node <= $nodes_number; $node++)
    {
        for(my $dest = 1; $dest <= $nodes_number; $dest++)
        {
            if($node != $dest)
            {
                print $node, ":", $dest, "\n";
            }
        }
    }

    exit 0;
}

###############################################################################
# Empty: Set of Isolated Nodes
###############################################################################

exit 0;
